#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/20 22:29
@File  : config.py
'''
class feishu(object):
    @staticmethod
    def webhook():
        ''' 飞书webhook的地址 '''
        return 'https://open.feishu.cn/open-apis/bot/v2/hook/xxxx'

class dingding(object):
    @staticmethod
    def webhook():
        ''' 钉钉webhook的地址 '''
        return 'https://oapi.dingtalk.com/robot/send?access_token=xxxxxxxx'

class email(object):
    @staticmethod
    def smtpServer():
        ''' smtp服务器IP或者域名
        163邮箱：http://help.163.com/09/1223/14/5R7P3QI100753VB8.html
        QQ邮箱：https://service.mail.qq.com/cgi-bin/help?subtype=1&id=28&no=369
        '''
        return 'smtp.163.com'

    @staticmethod
    def smtpPort():
        ''' smtp端口： '''
        return 25

    @staticmethod
    def username():
        ''' 邮箱登录账户 '''
        return 'username'

    @staticmethod
    def password():
        ''' 授权码，163和QQ的不是登录密码 '''
        return 'password'

    @staticmethod
    def isSSL():
        ''' 是否使用安全隧道 '''
        return False

class browser(object):
    @staticmethod
    def name():
        ''' 浏览器名称： chrome 或 firefox 或 ie '''
        return 'chrome'

class log(object):
    @staticmethod
    def path():
        ''' 日志存储的路径 '''
        return '.'

    @staticmethod
    def name():
        ''' 日志文件名 '''
        return 'auto_info_{time}.log'

    @staticmethod
    def rotation():
        ''' 日志存储规则：50 MB 或者 1 days '''
        return '1 days'


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/21 0:23
@File  : conftest.py
'''
from saf import pytest, logger, robot_send_message

@pytest.mark.hookwrapper
def pytest_runtest_makereport(item):
    """
    :param item:
    """
    outcome = yield
    report = outcome.get_result()
    logger.add(sink='auto_info_{time}.log', rotation='1 day')
    logger.info(report)
    if report.outcome == 'failed':
        robot_send_message(robot_name='feishu',
                           msg=f'测试脚本：{report.nodeid.split("::")[0]}\n测试用例：{report.nodeid.split("::")[1]}\n测试结果：{report.outcome}'
                           )

'''
关于一些Chrome浏览器的设置
chrome_options = Options()
# 去除"Chrome正在受到自动化测试软件的控制"弹出框信息
chrome_options.add_experimental_option('useAutomationExtension', False)
chrome_options.add_experimental_option('excludeSwitches', ['--enable-automation'])
# 启动最大化
chrome_options.add_argument('--start-maximized')
# 规避不必要的BUG出现
chrome_options.add_argument('--disable-gpu')
prefs = {}
prefs["credentials_enable_service"] = False
prefs["profile.password_manager_enabled"] = False
chrome_options.add_experimental_option("prefs", prefs)
# 无头（无界面）模式
# chrome_options.add_argument('--headless')
# 设置手机浏览器头（模拟手机端web）
# chrome_options.add_argument('User-Agent=Mozilla/5.0 (Linux; U; Android 8.1.0; zh-cn; BLA-AL00 Build/HUAWEIBLA-AL00) AppleWebKit/537.36 (KHTML, like Gecko) Version/4.0 Chrome/57.0.2987.132 MQQBrowser/8.9 Mobile Safari/537.36')
# 禁止图片
# chrome_options.add_argument('blink-settings=imagesEnabled=false')
# 指定chrome.exe路径
# chrome_options.binary_location = '*\\chrome.exe'
try:
    b = webdriver.Chrome(chrome_options=chrome_options)
'''


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/21 0:25
@File  : test_xiaobai_shop.py
'''
import pytest

def test_xiaobai_coupon():
    assert 1 == 1

def test_xiaobai_collection():
    assert 1 == 2


if __name__ == '__main__':
    pytest.main(['-s', '-v', 'test_xiaobai_shop.py'])


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/24 2:26
@File  : xiaobaicmd.py
'''
import os.path
from shutil import copytree
import click

@click.command()
@click.option('--template', '-t', default='web', type=click.Choice(['web', 'api', 'app']), nargs=1, help='创建自动化项目模板')
@click.option('--dirname', '-d', default='.', type=str, nargs=1, help='创建自动化项目模板存放的目录')
def main(template, dirname):
    if "web" == template.lower():
        copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)), r'..\web'),
                 os.path.join(os.path.abspath(dirname), 'web')
                 )
    elif "api" == template.lower():
        copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)), r'..\api'),
                 os.path.join(os.path.abspath(dirname), 'api')
                 )
    elif "app" == template.lower():
        copytree(os.path.join(os.path.dirname(os.path.abspath(__file__)), r'..\app'),
                 os.path.join(os.path.abspath(dirname), 'app')
                 )
    else:
        raise ("您输入的数据有误，有效范围：web 或 api 或 app")

# if __name__ == '__main__':
#     main()


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/20 22:47
@File  : __init__.py
'''
'''
PO模式：
    1、基础元素定位函数的封装（基础定位、键盘、鼠标事件，解决：定位不稳定问题）
    2、页面元素的定位（定位层）
    3、页面元素的操作（操作层）
    4、界面业务的操作（业务层）
    5、基于单元模块的用例（用例层）
'''
from saf import WebDriver, By
from time import sleep

def get_element(driver: WebDriver = None, loc: str = None, total_time: int = 30, step_time: float = 0.5):
    '''
    通用定位元素方法
    :param driver:          浏览器对象
    :param loc:             定位表达式
    :param total_time:      定位超时时间（单位：秒）
    :param step_time:       每次间隔定位时间（单位：秒）
    :return:
    '''
    while 1:
        if total_time > 0:
            if 0 == len(driver.find_elements(by=By.XPATH, value=loc)):
                sleep(step_time)
                total_time -= step_time
            else:
                return driver.find_element(by=By.XPATH, value=loc)
        else:
            break
    return None


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/20 22:22
@File  : __init__.py
'''
from typing import Union

# 将pip更新源设置为：https://pypi.douban.com/simple
# pip config set global.index-url https://pypi.douban.com/simple

from selenium import webdriver as selenium_webdriver
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys     # 键盘事件
from selenium.webdriver.common.action_chains import ActionChains  # 鼠标事件
# import os; os.system('pip install selenium -i https://pypi.douban.com/simple')

from webdriver_manager.chrome import ChromeDriverManager
# import os; os.system('pip install webdriver_manager -i https://pypi.douban.com/simple')

import pytest
# import os; os.system('pip install pytest pytest-HTML pytest-ordering -i https://pypi.douban.com/simple')

from loguru import logger
# import os; os.system('pip install loguru -i https://pypi.douban.com/simple')

# 当前版本的httpx，只有在异步请求时才略优于requests，此处可使用到发送消息使用
# from httpx import request
# import os; os.system('pip install httpx -i https://pypi.douban.com/simple')

from requests import request
# import os; os.system('pip install requests -i https://pypi.douban.com/simple')

from appium import webdriver as appium_webdriver
# import os; os.system('pip install Appium-Python-Client -i https://pypi.douban.com/simple')

from jmespath import search
# import os; os.system('pip install jmespath -i https://pypi.douban.com/simple')

from yaml import full_load
# import os; os.system('pip install pyyaml -i https://pypi.douban.com/simple')

import click
# import os; os.system('pip install click -i https://pypi.douban.com/simple')

from shutil import copytree
# import os; os.system('pip install shutil -i https://pypi.douban.com/simple')

try:
    from ddddocr import DdddOcr
    img2str = DdddOcr(show_ad=False).classification
except ImportError as e:
    import os; os.system('pip install ddddocr -i https://pypi.douban.com/simple')
    # 如果运行ddddocr运行报错更新opencv库：
    os.system('pip uninstall opencv-python')
    os.system('pip uninstall opencv-contrib-python')
    os.system('pip install opencv-contrib-python')
    os.system('pip install opencv-python')

from iniconfig import IniConfig
from saf.data.config import *

'''
调用飞书机器人发送飞书群消息
@feishu_help_document = https://open.feishu.cn/document/ukTMukTMukTM/ucTM5YjL3ETO24yNxkjN

调用钉钉机器人发送钉钉群消息
@dingding_help_document = https://open.dingtalk.com/document/group/custom-robot-access
'''

def send_message(
        url: str = None,
        msg: dict = None,
        assert_path: str = None,
        assert_value: Union[str, int] = None
):
    '''
    发送消息函数
    :param url:             WebHook的URL地址
    :param msg:             WebHook发送的信息
    :param assert_path:     判断内容的路径
    :param assert_value:    判断内容的值
    :return:
    '''
    if not url and not msg:
        raise ('url或者msg信息不能为空，必填内容！')
    else:
        response = request(
                        method='POST',
                        url=url,
                        headers={'Content-Type': 'application/json'},
                        json=msg
                    )
    if assert_path and assert_value:
        try:
            assert assert_value == search(assert_path, response.json())
        except AssertionError as e:
            raise ('信息发送失败！')

def feishu_webhook_send_message(msg:str = '本次测试结束'):
    '''
    使用飞书的webhook发送机器人消息
    :param msg:     需要发送的信息内容，
    :return:
    '''
    send_message(url=feishu.webhook(),
                 msg={
                     'msg_type': 'text',
                     'content': {
                         'text': msg
                     }
                 },
                 assert_path='StatusCode',
                 assert_value=0
    )

def dingding_webhook_send_message(msg:str = '本次测试结束'):
    '''
    使用钉钉的webhook发送机器人消息
    :param msg:     需要发送的信息内容，
    :return:
    '''
    send_message(url=dingding.webhook(),
                 msg={'msgtype': 'text',
                        'text': {
                         'content': msg
                        }
                 },
                 assert_path='errcode',
                 assert_value=0
    )

def robot_send_message(robot_name: str= 'feishu', msg: str = '本次测试结束'):
    ''' 统一调用机器人发送消息 '''
    if 'feishu' in robot_name:
        feishu_webhook_send_message(msg)
    if 'dingding' in robot_name:
        dingding_webhook_send_message(msg)


# if __name__ == '__main__':
#     robot_send_message(robot_name='feishu')


#! /usr/bin/env python
'''
@Author: xiaobaiTser
@Time  : 2022/8/24 1:37
@File  : image_utils.py
'''
''' 将图片验证码转为字符串的样例 '''
import logging
from saf import *

c = selenium_webdriver.Chrome(ChromeDriverManager(log_level=logging.ERROR).install())
c.get('http://121.62.63.73:10721/admin/login')

c.find_element(by=By.XPATH, value='//*[@class="pictrue"]').screenshot('1.png')
f = open('1.png', 'rb')
print(img2str(f.read()))
os.remove('1.png')